use tool_helper::Error;

pub struct VAGADPCM {
    data: [u32; 4]
}

impl VAGADPCM {
    const ADPCM_SAMPLES_PER_VAGADPCM:usize = 28;

    pub fn create() -> VAGADPCM {
        VAGADPCM{data: [0; 4]}
    }
}

pub struct MonoADPCMIterator<I: std::iter::Iterator<Item=Result<i16, hound::Error>>> {
    iter: I
}

impl<I:std::iter::Iterator<Item=Result<i16, hound::Error>>> MonoADPCMIterator<I>{
    pub fn create(iter: I) -> MonoADPCMIterator<I> {
        MonoADPCMIterator{iter}
    }
}

impl<I:std::iter::Iterator<Item=Result<i16, hound::Error>>> std::iter::Iterator for MonoADPCMIterator<I> {
    type Item = Result<[i16; VAGADPCM::ADPCM_SAMPLES_PER_VAGADPCM], Error>;

    fn next(&mut self) -> Option<Self::Item> {
        const STREAM_GONE_ERROR: &'static str = "Reading ADPCM sample failed";

        if let Some(next_sample) = self.iter.next() {
            let Ok(next_sample) = next_sample else {return Some(Err(Error::from_str(STREAM_GONE_ERROR)));};

            let mut sample = [0;VAGADPCM::ADPCM_SAMPLES_PER_VAGADPCM];
            sample[0] = next_sample;

            for idx in 1..VAGADPCM::ADPCM_SAMPLES_PER_VAGADPCM {
                let Ok(next_sample) = self.iter.next().unwrap_or(Ok(0)) else {return Some(Err(Error::from_str(STREAM_GONE_ERROR)));};
                sample[idx] = next_sample;
            }
            return Some(Ok(sample));
        }
        None
    }
}